﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Transformace
{
    public partial class Form1 : Form
    {
        private Bitmap bitmap;

        public Form1()
        {
            InitializeComponent();
            bitmap = new Bitmap(pictureBox1.Image);
        }

        private void Prahovani(float hodnota)
        {
            int i, j;
            Color c;
            Bitmap bmp = new Bitmap(pictureBox1.Image);

            for (i = 0; i < bmp.Width; i++)
                for (j = 0; j < bmp.Height; j++)
                {
                    c = bmp.GetPixel(i, j);
                    if (c.GetBrightness() > hodnota)
                        bmp.SetPixel(i, j, Color.Beige);
                    else
                        bmp.SetPixel(i, j, Color.Black);
                }
            pictureBox1.Image = bmp;
        }

        private void Nahoda()
        {
            int i, j;
            byte hodnota;
            Random rand = new Random();
            Color c;
            Bitmap bmp = new Bitmap(pictureBox1.Image);

            for (i = 0; i < bmp.Width; i++)
                for (j = 0; j < bmp.Height; j++)
                {
                    c = bmp.GetPixel(i, j);
                    hodnota = (byte)rand.Next(0, 255);
                    if (255 * c.GetBrightness() > hodnota)
                        bmp.SetPixel(i, j, Color.Beige);
                    else
                        bmp.SetPixel(i, j, Color.Black);
                }
            pictureBox1.Image = bmp;
        }

        private void Matice(bool disp)
        {
            int[,] Md = new int[4, 4] {{0, 12, 3, 15},
                                       {8, 4, 11, 7},
                                       {2, 14, 1, 13},
                                       {10, 6, 9, 5}};
            int[,] Mp = new int[4, 4] {{1, 5, 9, 2},
                                       {8, 12, 13, 6},
                                       {4, 15, 14, 10},
                                       {0, 11, 7, 3}};

            int i, j;
            Color c;
            Bitmap bmp = new Bitmap(pictureBox1.Image);
            int old_i;

            for (i = 0; i < bmp.Width; i++)
                for (j = 0; j < bmp.Height; j++)
                {
                    c = bmp.GetPixel(i, j);
                    old_i = (int)(255 * c.GetBrightness());
                    if (old_i > 16 * (disp ? Md[i % 4, j % 4] : Mp[i % 4, j % 4]))
                        bmp.SetPixel(i, j, Color.Beige);
                    else
                        bmp.SetPixel(i, j, Color.Black);
                }
            pictureBox1.Image = bmp;
        }

        private Bitmap FloydSteinberg(Bitmap Orig)
        {
            int[,] DithArr;
            int Vyska = Orig.Height;
            int Sirka = Orig.Width;
            DithArr = new int[Sirka, Vyska];
            int i, j, temp;
            Bitmap Novy = new Bitmap(Sirka, Vyska);

            for (j = 0; j < Vyska; j++)                 //nastaveni matice
                for (i = 0; i < Sirka; i++)
                {
                    Color pixel = Orig.GetPixel(i, j);
                    DithArr[i, j] = (int)(255 * pixel.GetBrightness());
                }

            for (j = 1; j < Vyska - 1; j++)             //vlastni vypocet 
                for (i = 1; i < Sirka - 1; i++)
                {
                    int qnt_err = 0;
                    if (DithArr[i, j] < 128)
                    {
                        qnt_err = DithArr[i, j] / 16;
                        DithArr[i, j] = 0;
                    }
                    else
                    {
                        qnt_err = (DithArr[i, j] - 255) / 16;
                        DithArr[i, j] = 1;
                    }
                    DithArr[i + 1, j] += (qnt_err * 7);
                    DithArr[i - 1, j + 1] += (qnt_err * 3);
                    DithArr[i, j + 1] += (qnt_err * 5);
                    DithArr[i + 1, j + 1] += (qnt_err);
                }

            for (j = 0; j < Vyska; j++)                 //novy obrazek
                for (i = 0; i < Sirka; i++)
                {
                    Color pixel = Orig.GetPixel(i, j);
                    temp = DithArr[i, j];
                    if (temp == 0) temp = 0; else temp = 255;
                    pixel = temp == 0 ? Color.Black : Color.Beige;
                    Novy.SetPixel(i, j, pixel);
                }
            return Novy;
        }

        private byte Nastav(byte hodnota, double k, double q)
        {
            double h = k * (double)hodnota + q;

            if (h < 0)
                return 0;
            else
            {
                if (h > 255)
                    return 255;
                else
                    return (byte)h;
            }
        }

        private void UpravObr()
        {
            int i, j;
            byte r, g, b;
            Color cold, cnew;
            Bitmap bmp = new Bitmap(pictureBox1.Image);
            double k = (double)trackBar1.Value / 10;
            double q = (double)trackBar2.Value * 10;
            for (i = 0; i < bmp.Width; i++)
                for (j = 0; j < bmp.Height; j++)
                {
                    cold = bmp.GetPixel(i, j);
                    r = Nastav(cold.R, k, q);
                    g = Nastav(cold.G, k, q);
                    b = Nastav(cold.B, k, q);
                    cnew = Color.FromArgb(r, g, b);
                    bmp.SetPixel(i, j, cnew);
                }
            pictureBox1.Image = bmp;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Prahovani(0.5f);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            pictureBox1.Image = bitmap;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            Nahoda();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            Matice(true);
        }

        private void button5_Click(object sender, EventArgs e)
        {
            Matice(false);
        }

        private void button6_Click(object sender, EventArgs e)
        {
            pictureBox1.Image = FloydSteinberg((Bitmap)pictureBox1.Image);
        }

        private void button7_Click(object sender, EventArgs e)
        {
            UpravObr();
        }
    }
}
